package com.sun.jmx.snmp;

import java.util.Vector;
import java.net.InetAddress;

public abstract class SnmpMsg implements SnmpDefinitions
{
    public int version;
    public byte[] data;
    public int dataLength;
    public InetAddress address;
    public int port;
    public SnmpSecurityParameters securityParameters;
    
    public SnmpMsg() {
        this.version = 0;
        this.data = null;
        this.dataLength = 0;
        this.address = null;
        this.port = 0;
        this.securityParameters = null;
    }
    
    public static int getProtocolVersion(byte[] array) throws SnmpStatusException {
        BerDecoder berDecoder;
        int fetchInteger;
        try {
            berDecoder = new BerDecoder(array);
            berDecoder.openSequence();
            fetchInteger = berDecoder.fetchInteger();
        }
        catch (BerException ex) {
            throw new SnmpStatusException("Invalid encoding");
        }
        try {
            berDecoder.closeSequence();
        }
        catch (BerException ex2) {}
        return fetchInteger;
    }
    
    public abstract int getRequestId(byte[] p0) throws SnmpStatusException;
    
    public abstract int encodeMessage(byte[] p0) throws SnmpTooBigException;
    
    public abstract void decodeMessage(byte[] p0, int p1) throws SnmpStatusException;
    
    public abstract void encodeSnmpPdu(SnmpPdu p0, int p1) throws SnmpStatusException, SnmpTooBigException;
    
    public abstract SnmpPdu decodeSnmpPdu() throws SnmpStatusException;
    
    public static String dumpHexBuffer(byte[] array, int i, int n) {
        StringBuffer sb = new StringBuffer(n << 1);
        int n2 = 1;
        int n3;
        int n4;
        for (n3 = i + n, ; i < n3; ++i) {
            n4 = (array[i] & 0xFF);
            sb.append(Character.forDigit(n4 >>> 4, 16));
            sb.append(Character.forDigit(n4 & 0xF, 16));
            if (++n2 % 16 == 0) {
                sb.append('\n');
                n2 = 1;
            }
            else {
                sb.append(' ');
            }
        }
        return sb.toString();
    }
    
    public String printMessage() {
        StringBuffer sb = new StringBuffer();
        sb.append("Version: ");
        sb.append(this.version);
        sb.append("\n");
        if (this.data == null) {
            sb.append("Data: null");
        }
        else {
            sb.append("Data: {\n");
            sb.append(dumpHexBuffer(this.data, 0, this.dataLength));
            sb.append("\n}\n");
        }
        return sb.toString();
    }
    
    public void encodeVarBindList(BerEncoder berEncoder, SnmpVarBind[] array) throws SnmpStatusException, SnmpTooBigException {
        int n = 0;
        try {
            berEncoder.openSequence();
            if (array != null) {
                for (int i = array.length - 1; i >= 0; --i) {
                    SnmpVarBind snmpVarBind = array[i];
                    if (snmpVarBind != null) {
                        berEncoder.openSequence();
                        this.encodeVarBindValue(berEncoder, snmpVarBind.value);
                        berEncoder.putOid(snmpVarBind.oid.longValue());
                        berEncoder.closeSequence();
                        ++n;
                    }
                }
            }
            berEncoder.closeSequence();
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            throw new SnmpTooBigException(n);
        }
    }
    
    void encodeVarBindValue(BerEncoder berEncoder, SnmpValue snmpValue) throws SnmpStatusException {
        if (snmpValue == null) {
            berEncoder.putNull();
        }
        else if ((snmpValue instanceof SnmpIpAddress)) {
            berEncoder.putOctetString(((SnmpIpAddress)snmpValue).byteValue(), 64);
        }
        else if ((snmpValue instanceof SnmpCounter)) {
            berEncoder.putInteger(((SnmpCounter)snmpValue).longValue(), 65);
        }
        else if ((snmpValue instanceof SnmpGauge)) {
            berEncoder.putInteger(((SnmpGauge)snmpValue).longValue(), 66);
        }
        else if ((snmpValue instanceof SnmpTimeticks)) {
            berEncoder.putInteger(((SnmpTimeticks)snmpValue).longValue(), 67);
        }
        else if ((snmpValue instanceof SnmpOpaque)) {
            berEncoder.putOctetString(((SnmpOpaque)snmpValue).byteValue(), 68);
        }
        else if ((snmpValue instanceof SnmpInt)) {
            berEncoder.putInteger(((SnmpInt)snmpValue).intValue());
        }
        else if ((snmpValue instanceof SnmpString)) {
            berEncoder.putOctetString(((SnmpString)snmpValue).byteValue());
        }
        else if ((snmpValue instanceof SnmpOid)) {
            berEncoder.putOid(((SnmpOid)snmpValue).longValue());
        }
        else if ((snmpValue instanceof SnmpCounter64)) {
            if (this.version == 0) {
                throw new SnmpStatusException("Invalid value for SNMP v1 : " + snmpValue);
            }
            berEncoder.putInteger(((SnmpCounter64)snmpValue).longValue(), 70);
        }
        else if ((snmpValue instanceof SnmpNull)) {
            int tag = ((SnmpNull)snmpValue).getTag();
            if (this.version == 0 && tag != 5) {
                throw new SnmpStatusException("Invalid value for SNMP v1 : " + snmpValue);
            }
            if (this.version == 1 && tag != 5 && tag != 128 && tag != 129 && tag != 130) {
                throw new SnmpStatusException("Invalid value " + snmpValue);
            }
            berEncoder.putNull(tag);
        }
        else {
            throw new SnmpStatusException("Invalid value " + snmpValue);
        }
    }
    
    public SnmpVarBind[] decodeVarBindList(BerDecoder berDecoder) throws BerException {
        berDecoder.openSequence();
        Vector<Object> vector = new Vector();
        while (berDecoder.cannotCloseSequence()) {
            SnmpVarBind obj = new SnmpVarBind();
            berDecoder.openSequence();
            obj.oid = new SnmpOid(berDecoder.fetchOid());
            obj.setSnmpValue(this.decodeVarBindValue(berDecoder));
            berDecoder.closeSequence();
            vector.addElement(obj);
        }
        berDecoder.closeSequence();
        SnmpVarBind[] anArray = new SnmpVarBind[vector.size()];
        vector.copyInto(anArray);
        return anArray;
    }
    
    SnmpValue decodeVarBindValue(BerDecoder berDecoder) throws BerException {
        int tag = berDecoder.getTag();
        SnmpInt snmpInt;
        switch (tag) {
            case 2: {
                try {
                    snmpInt = new SnmpInt(berDecoder.fetchInteger());
                    break;
                }
                catch (RuntimeException ex) {
                    throw new BerException();
                }
            }
            case 4: {
                try {
                    snmpInt = new SnmpString(berDecoder.fetchOctetString());
                    break;
                }
                catch (RuntimeException ex2) {
                    throw new BerException();
                }
            }
            case 6: {
                try {
                    snmpInt = new SnmpOid(berDecoder.fetchOid());
                    break;
                }
                catch (RuntimeException ex3) {
                    throw new BerException();
                }
            }
            case 5: {
                berDecoder.fetchNull();
                try {
                    snmpInt = new SnmpNull();
                    break;
                }
                catch (RuntimeException ex4) {
                    throw new BerException();
                }
            }
            case 64: {
                try {
                    snmpInt = new SnmpIpAddress(berDecoder.fetchOctetString(tag));
                    break;
                }
                catch (RuntimeException ex5) {
                    throw new BerException();
                }
            }
            case 65: {
                try {
                    snmpInt = new SnmpCounter(berDecoder.fetchIntegerAsLong(tag));
                    break;
                }
                catch (RuntimeException ex6) {
                    throw new BerException();
                }
            }
            case 66: {
                try {
                    snmpInt = new SnmpGauge(berDecoder.fetchIntegerAsLong(tag));
                    break;
                }
                catch (RuntimeException ex7) {
                    throw new BerException();
                }
            }
            case 67: {
                try {
                    snmpInt = new SnmpTimeticks(berDecoder.fetchIntegerAsLong(tag));
                    break;
                }
                catch (RuntimeException ex8) {
                    throw new BerException();
                }
            }
            case 68: {
                try {
                    snmpInt = new SnmpOpaque(berDecoder.fetchOctetString(tag));
                    break;
                }
                catch (RuntimeException ex9) {
                    throw new BerException();
                }
            }
            case 70: {
                if (this.version == 0) {
                    throw new BerException(1);
                }
                try {
                    snmpInt = new SnmpCounter64(berDecoder.fetchIntegerAsLong(tag));
                    break;
                }
                catch (RuntimeException ex10) {
                    throw new BerException();
                }
            }
            case 128: {
                if (this.version == 0) {
                    throw new BerException(1);
                }
                berDecoder.fetchNull(tag);
                snmpInt = SnmpVarBind.noSuchObject;
                break;
            }
            case 129: {
                if (this.version == 0) {
                    throw new BerException(1);
                }
                berDecoder.fetchNull(tag);
                snmpInt = SnmpVarBind.noSuchInstance;
                break;
            }
            case 130: {
                if (this.version == 0) {
                    throw new BerException(1);
                }
                berDecoder.fetchNull(tag);
                snmpInt = SnmpVarBind.endOfMibView;
                break;
            }
            default: {
                throw new BerException();
            }
        }
        return snmpInt;
    }
}
